from math import floor

from buils.build import Build
from buils.util import build_workers
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.knowledges import Knowledge
from sharpy.managers.roles import UnitTask
from sharpy.plans.tactics.zone_attack import AttackStatus
from sharpy.plans.zerg import *


class RoachLings(BuildOrder):
    def __init__(self):
        self.lings = ZergUnit(UnitTypeId.ZERGLING)
        self.roach = ZergUnit(UnitTypeId.ROACH, priority=True)
        super().__init__(self.roach, self.lings)

    async def execute(self) -> bool:
        self.roach.to_count = len(self.knowledge.unit_cache.own({UnitTypeId.ROACH})) + \
                              floor(self.knowledge.available_gas / 25)
        self.lings.to_count = 400

        return await super().execute()


class LingRoachAttack(Build):
    def __init__(self, knowledge: Knowledge):
        super()
        self.blocking = True
        self.time_to_done = 6*60
        self.knowledge = knowledge

    def unblock(self):
        self.blocking = False
        self.knowledge.chat_manager.chat_debug("Done rushing")

    def attack(self):
        self.knowledge.roles.set_tasks(UnitTask.Attacking, self.knowledge.unit_cache.own(
            {UnitTypeId.ROACH, UnitTypeId.ZERGLING}))
        zone_attack = self.knowledge.get_manager(PlanZoneAttack)
        zone_attack.status = AttackStatus.Attacking

    def build(self):
        return SequentialList(
            # Opener
            Step(Supply(17), Expand(2)),
            StepBuildGas(1, Supply(18)),
            Step(
                Supply(17),
                PositionBuilding(UnitTypeId.SPAWNINGPOOL, DefensePosition.BehindMineralLineLeft, 0),
            ),
            ActUnit(UnitTypeId.ZERGLING, UnitTypeId.LARVA, 4),
            ZergUnit(UnitTypeId.QUEEN, 2),
            Step(All([UnitReady(UnitTypeId.SPAWNINGPOOL), Gas(100)]), Tech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
            Step(All([UnitReady(UnitTypeId.SPAWNINGPOOL), Gas(50)]), ActBuilding(UnitTypeId.ROACHWARREN)),

            # Build some workers if it's safe
            # Start with 6 roaches when roachwarren finishes
            # Build some lings if roachwarren is not ready if needed
            Step(None, ZergUnit(UnitTypeId.DRONE, 24), skip=RequireCustom(lambda k: not build_workers(k))),
            Step(UnitReady(UnitTypeId.ROACHWARREN), ZergUnit(UnitTypeId.ROACH, 6), skip=RequireCustom(lambda k: not len(k.unit_cache.own(UnitTypeId.ROACHWARREN).ready) > 0 or k.ai.time > self.time_to_done)),
            Step(None, ZergUnit(UnitTypeId.ZERGLING), skip=RequireCustom(lambda k: len(k.unit_cache.own(UnitTypeId.ROACHWARREN).ready) > 0 or k.ai.time > self.time_to_done)),
            Step(UnitReady(UnitTypeId.ROACHWARREN), RoachLings(), skip=Time(self.time_to_done)),
            Step(Time(3*60+30), ActCustom(lambda: self.attack()), skip=Time(self.time_to_done)),
            Step(Time(self.time_to_done), ActCustom(lambda: self.unblock()), skip=Time(self.time_to_done + 5)),
            Step(Time(self.time_to_done), Expand(3)),
        )

    def tech(self) -> SequentialList:
        return SequentialList([])

    def units(self):
        return SequentialList([])
