from buils.build import Build
from buils.midgame.earlyHydra import LingHydra
from buils.util import build_workers, ranged_upgrades, melee_upgrades
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


class HydraCorruptor(BuildOrder):
    def __init__(self):
        self.corruptor = ZergUnit(UnitTypeId.CORRUPTOR)
        self.hydra = ZergUnit(UnitTypeId.HYDRALISK)
        super().__init__(self.corruptor, self.hydra)

    async def execute(self) -> bool:
        return await super().execute()


class HydraCorruptorBroodLord(BuildOrder):
    def __init__(self):
        self.corruptor = ZergUnit(UnitTypeId.CORRUPTOR)
        self.broodLord = ZergUnit(UnitTypeId.BROODLORD, to_count=10)
        self.hydra = ZergUnit(UnitTypeId.HYDRALISK)
        super().__init__(self.corruptor, self.broodLord, self.hydra)

    async def execute(self) -> bool:
        return await super().execute()


class BroodLordBuild(Build):
    def build(self):
        return SequentialList(
            Step(Time(10 * 60), None),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.HATCHERY), RequiredGas(100)]), MorphLair(), skip=RequiredUnitReady(UnitTypeId.HIVE)),
            ActBuilding(UnitTypeId.INFESTATIONPIT),
            Step(RequiredSupply(50), ActBuilding(UnitTypeId.HYDRALISKDEN)),
            Step(RequiredUnitReady(UnitTypeId.LAIR), ActBuilding(UnitTypeId.SPIRE)),
            MorphHive(),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.HIVE), RequiredUnitReady(UnitTypeId.SPIRE)]), MorphGreaterSpire()),
        )

    def tech(self) -> SequentialList:
        return SequentialList(
            Step(build_workers, None, skip=Time(8 * 60)),
            # Upgrades after a while
            Step(Time(60 * 6), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
            Step(None, SequentialList(
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL3)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL3)),
            )),
            ranged_upgrades,
            melee_upgrades,
        )

    def units(self):
        return SequentialList(
            Step(None, HydraCorruptorBroodLord(), skip_until=RequiredAll([RequiredUnitReady(UnitTypeId.GREATERSPIRE), RequiredUnitReady(UnitTypeId.HYDRALISKDEN)])),
            Step(None, HydraCorruptor(), skip_until=RequiredAll([RequiredUnitReady(UnitTypeId.SPIRE), RequiredUnitReady(UnitTypeId.HYDRALISKDEN)])),
            ZergUnit(UnitTypeId.ZERGLING)
        )

