from buils.old.LurkerHydra import LingsAndHydras
from buils.build import Build
from buils.util import build_workers, ranged_upgrades, melee_upgrades
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


class LingAndHydrasAndLurkers(BuildOrder):
    def __init__(self):
        self.lurkers = ZergUnit(UnitTypeId.LURKERMP, priority=True)
        self.hydras = ZergUnit(UnitTypeId.HYDRALISK, priority=True)
        self.lings = ZergUnit(UnitTypeId.ZERGLING)

        super().__init__(self.lurkers, self.hydras, self.lings)

    async def execute(self) -> bool:
        self.lurkers.to_count = self.get_count(UnitTypeId.HYDRALISK) / 3 + 1
        self.hydras.to_count = self.get_count(UnitTypeId.ZERGLING) * 1.2
        return await super().execute()


class LurkersBuild(Build):
    def build(self):
        return SequentialList(
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(RequiredSupply(50), ActBuilding(UnitTypeId.HYDRALISKDEN)),
            Step(RequiredSupply(50), ActBuilding(UnitTypeId.LURKERDENMP)),
            ActBuilding(UnitTypeId.INFESTATIONPIT),
            Step(Time(10 * 60), None),
            MorphHive(),
            Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.LURKERRANGE)),
            Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGLINGATTACKSPEED)),
        )

    def tech(self) -> SequentialList:
        return SequentialList(
            # Upgrades after a while
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), RequiredGas(100)]), ActTech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
            Step(Time(60 * 6), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
            ranged_upgrades,
            melee_upgrades,
        )

    def units(self):
        return SequentialList(
            # Units
            Step(None, LingAndHydrasAndLurkers(), skip_until=RequiredUnitReady(UnitTypeId.LURKERDENMP)),
            Step(None, LingsAndHydras(), skip_until=RequiredUnitReady(UnitTypeId.HYDRALISKDEN)),
            ZergUnit(UnitTypeId.ZERGLING),
        )

