from buils.build import Build
from buils.util import build_workers, ranged_upgrades, melee_upgrades
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


class LingHydra(BuildOrder):
    def __init__(self):
        self.hydra = ZergUnit(UnitTypeId.HYDRALISK)
        self.ling = ZergUnit(UnitTypeId.ZERGLING)
        super().__init__(self.hydra, self.ling)

    async def execute(self) -> bool:
        self.hydra.to_count = self.get_count(UnitTypeId.ZERGLING) * 0.4
        return await super().execute()


class EarlyHydra(Build):
    def build(self):
        return SequentialList(
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), RequiredGas(100)]), ActTech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.HATCHERY), RequiredGas(100)]), MorphLair(), skip=RequiredUnitReady(UnitTypeId.HIVE)),
            Step(RequiredUnitReady(UnitTypeId.LAIR), ActBuilding(UnitTypeId.HYDRALISKDEN)),
            Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEMUSCULARAUGMENTS)),
            Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEGROOVEDSPINES)),
        )

    def tech(self) -> SequentialList:
        return SequentialList(
            # Upgrades after a while
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(Time(60 * 6), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
            ranged_upgrades,
        )

    def units(self):
        return SequentialList(
            Step(None, LingHydra(), skip_until=RequiredUnitReady(UnitTypeId.HYDRALISKDEN)),
            Step(None, ZergUnit(UnitTypeId.ZERGLING), skip=RequiredUnitReady(UnitTypeId.HYDRALISKDEN)),
        )

