from buils.build import Build
from buils.util import build_workers, ranged_upgrades, melee_upgrades
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


class LingBane(BuildOrder):
    def __init__(self):
        self.bling = ZergUnit(UnitTypeId.BANELING)
        self.ling = ZergUnit(UnitTypeId.ZERGLING)
        super().__init__(self.bling, self.ling)

    async def execute(self) -> bool:
        return await super().execute()


class LingBaneBuild(Build):
    def build(self):
        return SequentialList(
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), RequiredGas(100)]), ActTech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), RequiredGas(50)]), ActBuilding(UnitTypeId.BANELINGNEST)),

            Step(build_workers, None, skip=Time(8 * 60)),
            Step(RequiredUnitReady(UnitTypeId.BANELINGNEST), MorphLair()),
            Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.CENTRIFICALHOOKS)),
        )

    def tech(self) -> SequentialList:
        return SequentialList(
            # Upgrades after a while
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(Gas(200), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
            melee_upgrades,
        )

    def units(self):
        return SequentialList(
            Step(None, LingBane(), skip_until=RequiredUnitReady(UnitTypeId.BANELINGNEST)),
            ZergUnit(UnitTypeId.ZERGLING)
        )

