from buils.build import Build
from buils.util import build_workers, ranged_upgrades
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


class RoachAndHydras(BuildOrder):
    def __init__(self):

        self.hydras = ZergUnit(UnitTypeId.HYDRALISK, priority=True)
        self.roach = ZergUnit(UnitTypeId.ROACH)
        super().__init__(self.hydras, self.roach)

    async def execute(self) -> bool:
        self.hydras.to_count = self.get_count(UnitTypeId.ROACH) * 1.2
        return await super().execute()

class RoachHydra(Build):
    def build(self):
        return SequentialList(
            # Tech
            Step(RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), ActBuilding(UnitTypeId.ROACHWARREN), skip=UnitExists(UnitTypeId.ROACHWARREN)),
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.HATCHERY), RequiredGas(100)]), MorphLair(),
                 skip=RequiredUnitReady(UnitTypeId.HIVE)),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.ROACHWARREN), RequiredUnitReady(UnitTypeId.LAIR)]),
                 ActTech(UpgradeId.GLIALRECONSTITUTION)),
            Step(None, ActBuilding(UnitTypeId.HYDRALISKDEN)),
            Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEMUSCULARAUGMENTS)),
            Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEGROOVEDSPINES)),
        )

    def tech(self) -> SequentialList:
        return SequentialList(
            # Upgrades after a while
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(All([Gas(200), UnitReady(UnitTypeId.DRONE, 30)]), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
            ranged_upgrades,
        )

    def units(self):
        return SequentialList(
            Step(None, RoachAndHydras(), skip_until=RequiredUnitReady(UnitTypeId.HYDRALISKDEN)),
            Step(None, ZergUnit(UnitTypeId.ROACH), skip_until=RequiredUnitReady(UnitTypeId.ROACHWARREN)),
            ZergUnit(UnitTypeId.ZERGLING)
        )
