from buils.build import Build
from buils.util import build_workers, ranged_upgrades
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


class RoachAndRavager(BuildOrder):
    def __init__(self):
        self.ravager = ZergUnit(UnitTypeId.RAVAGER)
        self.roach = ZergUnit(UnitTypeId.ROACH)
        super().__init__(self.ravager, self.roach)

    async def execute(self) -> bool:
        is_under_attack = False
        for zone in self.knowledge.zone_manager.expansion_zones:
            if zone.is_ours and zone.is_under_attack and len(zone.assaulting_enemies.exclude_type({UnitTypeId.BATTLECRUISER, UnitTypeId.ORACLE, UnitTypeId.OVERLORD, UnitTypeId.OVERSEER, UnitTypeId.ZERGDROPPOD})) > 0:
                is_under_attack = True

        if not is_under_attack:
            self.ravager.to_count = len(self.cache.own(UnitTypeId.ROACH))*2
        else:
            self.ravager.to_count = 0

        return await super().execute()

class AntiAir(BuildOrder):
    def __init__(self):
        self.corruptor = ZergUnit(UnitTypeId.CORRUPTOR)
        super().__init__(self.corruptor)

    def count(self, type: UnitTypeId):
        return self.knowledge.enemy_units_manager.unit_count(type)

    async def execute(self) -> bool:
        self.corruptor.to_count = \
            self.count(UnitTypeId.BATTLECRUISER) * 4 + \
            self.count(UnitTypeId.MUTALISK) * 0.75 + \
            self.count(UnitTypeId.CARRIER) * 3 + \
            self.count(UnitTypeId.TEMPEST) * 2 + \
            self.count(UnitTypeId.VIKING) * 1.5 + \
            self.count(UnitTypeId.LIBERATOR) * 1.5

        return await super().execute()


class RoachPlay(Build):
    def build(self):
        return SequentialList(
            # Tech
            Step(All([UnitReady(UnitTypeId.SPAWNINGPOOL), Gas(50)]), ActBuilding(UnitTypeId.ROACHWARREN)),
            Step(All([UnitReady(UnitTypeId.SPAWNINGPOOL), Gas(100)]), Tech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(All([UnitReady(UnitTypeId.HATCHERY), Gas(100)]), MorphLair(),
                 skip=UnitReady(UnitTypeId.HIVE)),
            Step(All([UnitReady(UnitTypeId.ROACHWARREN), UnitReady(UnitTypeId.LAIR)]),
                 Tech(UpgradeId.GLIALRECONSTITUTION)),
            Step(
                None,
                StepBuildGas(4),
                skip_until=UnitExists(UnitTypeId.DRONE, 36, include_pending=True)
            ),
            Step(UnitReady(UnitTypeId.DRONE, 55), Expand(4)),

            # Anti air response
            SequentialList([
                Step(
                    All([
                        RequireCustom(lambda k: len(k.known_enemy_units({UnitTypeId.BATTLECRUISER, UnitTypeId.MUTALISK, UnitTypeId.CARRIER, UnitTypeId.TEMPEST, UnitTypeId.LIBERATOR})) > 0),
                        UnitReady(UnitTypeId.DRONE, 66)]),
                    ActBuilding(UnitTypeId.SPIRE)),
                AntiAir(),
            ]),
        )

    def tech(self) -> SequentialList:
        return SequentialList(
            # Upgrades after a while
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(All([Gas(200), UnitReady(UnitTypeId.DRONE, 30)]),
                 ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
            ranged_upgrades,
        )

    def units(self):
        return SequentialList(
            Step(None, RoachAndRavager(), skip_until=UnitReady(UnitTypeId.ROACHWARREN), skip=RequireCustom(lambda k: k.available_gas < 100 and k.available_mineral > 600)),
            ZergUnit(UnitTypeId.ZERGLING)
        )
