from buils.build import Build
from buils.midgame.earlyHydra import LingHydra
from buils.midgame.roachhydra import RoachAndHydras
from buils.util import build_workers, ranged_upgrades, melee_upgrades
from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


class RoachCorruptor(BuildOrder):
    def __init__(self):
        self.corruptor = ZergUnit(UnitTypeId.CORRUPTOR, priority=True)
        self.roach = ZergUnit(UnitTypeId.ROACH)
        super().__init__(self.corruptor, self.roach)

    async def execute(self) -> bool:
        self.corruptor.to_count = self.get_count(UnitTypeId.ROACH) * 0.6
        return await super().execute()

class SpireBuild(Build):
    def build(self):
        return SequentialList(
            Step(RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), ActBuilding(UnitTypeId.ROACHWARREN)),
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.HATCHERY), RequiredGas(100)]), MorphLair(), skip=RequiredUnitReady(UnitTypeId.HIVE)),
            Step(RequiredUnitReady(UnitTypeId.LAIR), ActBuilding(UnitTypeId.SPIRE)),
        )

    def tech(self) -> SequentialList:
        return SequentialList(
            Step(build_workers, None, skip=Time(8 * 60)),
            Step(RequiredAll([RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), RequiredGas(100)]), ActTech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
            # Upgrades after a while
            Step(Time(60 * 6), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
            ranged_upgrades,
            SequentialList(
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL3)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL3)),
            ),
        )

    def units(self):
        return SequentialList(
            Step(None, ZergUnit(UnitTypeId.CORRUPTOR, to_count=8), skip_until=RequiredUnitReady(UnitTypeId.SPIRE)),
            Step(None, RoachAndHydras(), skip_until=RequiredUnitReady(UnitTypeId.HYDRALISKDEN)),
            Step(None, RoachCorruptor(), skip_until=RequiredUnitReady(UnitTypeId.SPIRE)),
            Step(None, ZergUnit(UnitTypeId.ROACH), skip_until=RequiredUnitReady(UnitTypeId.ROACHWARREN)),
            ZergUnit(UnitTypeId.ZERGLING)
        )

