
from sc2 import UnitTypeId, Race
from sc2.ids.upgrade_id import UpgradeId
from sc2.unit import Unit
from sharpy.knowledges import Knowledge
from sharpy.plans.zerg import *

class Lings(BuildOrder):
    def __init__(self):
        self.lings = ZergUnit(UnitTypeId.ZERGLING)
        super().__init__(self.lings)

    async def execute(self) -> bool:
        self.lings.to_count = self.get_count(UnitTypeId.ZERGLING) + 2
        return await super().execute()


class LingsAndHydras(BuildOrder):
    def __init__(self):
        self.hydras = ZergUnit(UnitTypeId.HYDRALISK, priority=True)
        self.lings = ZergUnit(UnitTypeId.ZERGLING)
        super().__init__(self.hydras, self.lings)

    async def execute(self) -> bool:
        self.hydras.to_count = self.get_count(UnitTypeId.ZERGLING) * 1.0
        self.lings.to_count = self.get_count(UnitTypeId.ZERGLING)
        return await super().execute()


class LingAndHydrasAndLurkers(BuildOrder):
    def __init__(self):
        self.lurkers = ZergUnit(UnitTypeId.LURKERMP, priority=True)
        self.hydras = ZergUnit(UnitTypeId.HYDRALISK, priority=True)
        self.lings = ZergUnit(UnitTypeId.ZERGLING)

        super().__init__(self.lurkers, self.hydras, self.lings)

    async def execute(self) -> bool:
        self.lurkers.to_count = self.get_count(UnitTypeId.HYDRALISK) / 3 + 1
        self.hydras.to_count = self.get_count(UnitTypeId.ZERGLING) * 1.2
        return await super().execute()


class LurkerBuild(BuildOrder):
    def __init__(self):

        super().__init__(
            SequentialList(
                # Overlords
                Step(RequiredSupply(13), ZergUnit(UnitTypeId.OVERLORD, 2, priority=True)),
                Step(RequiredSupply(20), ZergUnit(UnitTypeId.OVERLORD, 3, priority=True)),
                AutoOverLord(),
            ),
            SequentialList(
                # Ling zealot defence
                Step(RequiredAny(
                    [EnemyUnitExists(UnitTypeId.ZERGLING, count=5), EnemyUnitExists(UnitTypeId.ZEALOT, count=5)]),
                     ActBuilding(UnitTypeId.BANELINGNEST)),
                ZergUnit(UnitTypeId.BANELING, to_count=8),
                ActTech(UpgradeId.CENTRIFICALHOOKS),
            ),
            SequentialList(
                # Opener
                Step(RequiredSupply(17), ActExpand(2)),
                Step(
                    RequiredSupply(17),
                    PositionBuilding(UnitTypeId.SPAWNINGPOOL, DefensePosition.BehindMineralLineLeft, 0),
                ),
                StepBuildGas(1, RequiredSupply(16)),
                ActUnit(UnitTypeId.ZERGLING, UnitTypeId.LARVA, 4),
                ZergUnit(UnitTypeId.QUEEN, 2),
                Step(
                    UnitExists(UnitTypeId.DRONE, 32, include_pending=True),
                    StepBuildGas(4),
                ),
                Step(self.build_workers, None, skip=Time(5 * 60)),
                Step(Time(4 * 60), ActExpand(3)),
                Step(None, StepBuildGas(6)),
                ZergUnit(UnitTypeId.QUEEN, 4),
            ),
            SequentialList(
                # Workers
                Step(self.build_workers, ZergUnit(UnitTypeId.DRONE, 80), skip=self.max_workers_reached),
            ),
            Step(RequiredGas(90), ActTech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
            SequentialList(
                Step(RequireCustom(lambda k: k.enemy_units_manager.enemy_cloak_trigger), MorphLair()),
                Step(RequiredUnitReady(UnitTypeId.LAIR), MorphOverseer(2)),
            ),
            SequentialList(
                # Tech
                Step(self.build_workers, None, skip=Time(8 * 60)),
                Step(RequiredUnitReady(UnitTypeId.HATCHERY), MorphLair(), skip=RequiredUnitReady(UnitTypeId.HIVE)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActBuilding(UnitTypeId.HYDRALISKDEN)),
                Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEMUSCULARAUGMENTS)),
                Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEGROOVEDSPINES)),
                Step(RequiredSupply(50), ActBuilding(UnitTypeId.LURKERDENMP)),
                ActBuilding(UnitTypeId.INFESTATIONPIT),
                Step(Time(10 * 60), None),
                MorphHive(),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.LURKERRANGE)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGLINGATTACKSPEED)),
                Step(Time(15 * 60), None),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActBuilding(UnitTypeId.ULTRALISKCAVERN)),
            ),
            SequentialList(
                # Upgrades
                Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
                Step(self.build_workers, None, skip=Time(8 * 60)),
                Step(RequiredUnitReady(UnitTypeId.EVOLUTIONCHAMBER), ActTech(UpgradeId.ZERGMELEEWEAPONSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.EVOLUTIONCHAMBER), ActTech(UpgradeId.ZERGMISSILEWEAPONSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.EVOLUTIONCHAMBER), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.ZERGMELEEWEAPONSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.ZERGMISSILEWEAPONSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGMELEEWEAPONSLEVEL3)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGMISSILEWEAPONSLEVEL3)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL3)),
            ),
            SequentialList(
                # Some standard air defence
                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 40),
                     PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=0)),
                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 39),
                     PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=1,
                                      to_count=2)),
                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 38),
                     PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=2,
                                      to_count=3)),
            ),
            SequentialList(
                # Some vipers are good to have
                Step(self.build_workers, None, skip=Time(8 * 60)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ZergUnit(UnitTypeId.VIPER, to_count=3)),
                Step(RequiredUnitReady(UnitTypeId.ULTRALISKCAVERN), ZergUnit(UnitTypeId.ULTRALISK, to_count=5)),
            ),
            Step(
                RequiredMinerals(400),
                ZergUnit(UnitTypeId.QUEEN, 8),
                skip=RequireCustom(lambda knowledge: knowledge.enemy_race == Race.Terran)
            ),
            SequentialList(
                # more bases, because too little gas
                Step(Time(7 * 60), None),
                Step(None, ActExpand(10), skip=self.skip_new_base),
                StepBuildGas(20, skip=RequireCustom(lambda k: len(self.cache.own({UnitTypeId.DRONE})) < 32 or len(self.cache.own({UnitTypeId.HATCHERY, UnitTypeId.LAIR, UnitTypeId.HIVE}).ready) * 2 < len(self.cache.own(
                    {UnitTypeId.EXTRACTOR, UnitTypeId.EXTRACTORRICH})))),
                Step(None, ZergUnit(UnitTypeId.QUEEN, 10), RequireCustom(lambda k: len(self.cache.own({UnitTypeId.HATCHERY, UnitTypeId.LAIR, UnitTypeId.HIVE})) < len(self.cache.own({UnitTypeId.QUEEN})))),
                Step(self.build_workers, ZergUnit(UnitTypeId.DRONE, 80), skip=RequireCustom(lambda k: self.max_workers_reached(k) or (k.available_gas < 25 and k.available_mineral > 50))),
            ),
            Step(
                None,
                SequentialList(
                    # Units
                    Step(None, LingAndHydrasAndLurkers(), skip_until=RequiredUnitReady(UnitTypeId.LURKERDENMP)),
                    Step(None, LingsAndHydras(), skip_until=RequiredUnitReady(UnitTypeId.HYDRALISKDEN)),
                    ZergUnit(UnitTypeId.ZERGLING),
                ),
            ),
        )

    def build_workers(self, knowledge: Knowledge) -> bool:

        for zone in knowledge.zone_manager.expansion_zones:
            if zone.is_ours and zone.is_under_attack:
                return False

        return knowledge.game_analyzer.army_can_survive

    def max_workers_reached(self, knowledge: Knowledge) -> bool:
        count = 1
        for townhall in self.ai.townhalls:  # type: Unit
            if townhall.is_ready:
                count += townhall.ideal_harvesters
            elif townhall.is_ready > 0.6:
                count += 16
        for gas in self.ai.gas_buildings:  # type: Unit
            count += gas.ideal_harvesters

        if len(knowledge.unit_cache.own(UnitTypeId.DRONE)) >= count:
            return True

        return False

    def skip_new_base(self, knowledge: Knowledge) -> bool:
        count = 1
        for townhall in self.ai.townhalls:  # type: Unit
            if townhall.is_ready:
                count += townhall.ideal_harvesters
            else:
                count += 16
        for gas in self.ai.gas_buildings:  # type: Unit
            count += gas.ideal_harvesters

        count += 16 * self.ai.already_pending(UnitTypeId.HATCHERY)

        if (len(knowledge.unit_cache.own(UnitTypeId.DRONE)) > count or knowledge.available_mineral > 500) and len(knowledge.unit_cache.own(UnitTypeId.DRONE)) > 28:
            return  not knowledge.game_analyzer.army_can_survive

        return True