
from sc2 import UnitTypeId, Race
from sc2.ids.upgrade_id import UpgradeId
from sc2.unit import Unit
from sharpy.knowledges import Knowledge
from sharpy.plans.zerg import *

class LingsBanes(BuildOrder):
    def __init__(self):
        self.banes = ZergUnit(UnitTypeId.BANELING, priority=True)
        self.lings = ZergUnit(UnitTypeId.ZERGLING)
        super().__init__(self.banes, self.lings)

    async def execute(self) -> bool:
        self.banes.to_count = self.get_count(UnitTypeId.ZERGLING) * 0.3
        return await super().execute()


class LingsBanesMuta(BuildOrder):
    def __init__(self):
        self.muta = ZergUnit(UnitTypeId.MUTALISK)
        self.banes = ZergUnit(UnitTypeId.BANELING)
        self.lings = ZergUnit(UnitTypeId.ZERGLING)

        super().__init__(self.muta, self.banes, self.lings)

    async def execute(self) -> bool:
        self.muta.to_count = self.get_count(UnitTypeId.ZERGLING) * 3
        self.banes.to_count = self.get_count(UnitTypeId.ZERGLING) * 0.3
        return await super().execute()



class MutaLingBane(BuildOrder):
    def __init__(self):
        heavy_gas = SequentialList(
            [
                Step(
                    None,
                    StepBuildGas(2),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(20, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(3),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(30, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(4),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(40, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(5),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(50, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(6),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(60, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(7),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(65, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(8),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(70, supply_type=SupplyType.Workers),
                ),
            ]
        )

        super().__init__(
            SequentialList(
                # Overlords
                Step(UnitExists(UnitTypeId.DRONE, 13, include_pending=True),
                     ZergUnit(UnitTypeId.OVERLORD, 2, priority=True)),
                Step(UnitExists(UnitTypeId.DRONE, 20, include_pending=True),
                     ZergUnit(UnitTypeId.OVERLORD, 3, priority=True)),
                AutoOverLord(),
            ),
            SequentialList(
                # Opener
                Step(RequiredSupply(17), ActExpand(2)),
                Step(
                    RequiredSupply(17),
                    PositionBuilding(UnitTypeId.SPAWNINGPOOL, DefensePosition.BehindMineralLineLeft, 0),
                ),
                StepBuildGas(1, RequiredSupply(22)),
                ActUnit(UnitTypeId.ZERGLING, UnitTypeId.LARVA, 4),
                ZergUnit(UnitTypeId.QUEEN, 2),
                Step(self.build_workers, None, skip=Time(8 * 60)),
                Step(RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), ActBuilding(UnitTypeId.BANELINGNEST)),
                Step(RequiredUnitReady(UnitTypeId.HATCHERY), MorphLair(), skip=RequiredUnitReady(UnitTypeId.HIVE)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActBuilding(UnitTypeId.SPIRE)),
                Step(
                    UnitExists(UnitTypeId.DRONE, 32, include_pending=True),
                    StepBuildGas(4),
                ),
                Step(RequiredUnitReady(UnitTypeId.LAIR), MorphOverseer(2)),
            ),
            SequentialList(
                # Workers
                Step(self.build_workers, ZergUnit(UnitTypeId.DRONE, 22)),
                Step(self.build_workers, ZergUnit(UnitTypeId.DRONE, 44), skip=self.max_workers_reached),
            ),
            SequentialList(
                Step(RequiredGas(90), ActTech(UpgradeId.ZERGLINGMOVEMENTSPEED)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), MorphOverseer(2)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.CENTRIFICALHOOKS)),
            ),
            SequentialList(
                Step(RequireCustom(lambda k: k.enemy_units_manager.enemy_cloak_trigger), MorphLair()),
                Step(RequiredUnitReady(UnitTypeId.LAIR), MorphOverseer(2)),
            ),
            SequentialList(
                # Tech
                Step(RequiredSupply(120), ActBuilding(UnitTypeId.INFESTATIONPIT)),
                MorphHive(),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGLINGATTACKSPEED)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActBuilding(UnitTypeId.ULTRALISKCAVERN)),
            ),
            SequentialList(
                # Upgrades
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
                Step(RequiredUnitReady(UnitTypeId.EVOLUTIONCHAMBER), ActTech(UpgradeId.ZERGMELEEWEAPONSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.EVOLUTIONCHAMBER), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.ZERGMELEEWEAPONSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGMELEEWEAPONSLEVEL3)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL3)),
            ),
            SequentialList(
                # Upgrades
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERWEAPONSLEVEL3)),
                Step(RequiredUnitReady(UnitTypeId.SPIRE), ActTech(UpgradeId.ZERGFLYERARMORSLEVEL3)),
            ),
            SequentialList(
                # Some standard air defence
                Step(self.build_workers, None, skip=Time(8 * 60)),
                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 40),
                     PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=0)),
                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 39),
                     PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=1,
                                      to_count=2)),
                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 38),
                     PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=2,
                                      to_count=3)),
            ),
            SequentialList(
                # Some vipers are good to have
                Step(RequiredUnitReady(UnitTypeId.HIVE), ZergUnit(UnitTypeId.VIPER, to_count=3)),
                Step(RequiredUnitReady(UnitTypeId.ULTRALISKCAVERN), ZergUnit(UnitTypeId.ULTRALISK, to_count=5)),
            ),
            #SequentialList(
            #    Step(
            #        RequiredUnitReady(UnitTypeId.LAIR),
            #        ZergUnit(UnitTypeId.QUEEN, 8),
            #        skip=RequireCustom(lambda knowledge: knowledge.enemy_race == Race.Terran)
            #    ),
            #),

            SequentialList(
                # more bases, because too little gas
                Step(None, ActExpand(10), skip=RequireCustom(
                    lambda k: len(self.cache.own({UnitTypeId.DRONE})) < 28 or k.available_mineral < 500)),
                StepBuildGas(20, skip=RequireCustom(lambda k: len(self.cache.own({UnitTypeId.DRONE})) < 32 or len(
                    self.cache.own({UnitTypeId.HATCHERY, UnitTypeId.LAIR, UnitTypeId.HIVE})) * 2 < len(self.cache.own(
                    {UnitTypeId.EXTRACTOR, UnitTypeId.EXTRACTORRICH})))),
                Step(None, ZergUnit(UnitTypeId.QUEEN, 10), RequireCustom(
                    lambda k: len(self.cache.own({UnitTypeId.HATCHERY, UnitTypeId.LAIR, UnitTypeId.HIVE})) < len(
                        self.cache.own({UnitTypeId.QUEEN})))),
                Step(None, ZergUnit(UnitTypeId.DRONE, 80), skip=RequireCustom(
                    lambda k: not self.build_workers(k) or self.max_workers_reached(self.knowledge) or (
                                k.available_gas < 25 and k.available_mineral > 50))),
            ),
            Step(
                None,
                SequentialList(
                    # Units
                    Step(None, LingsBanesMuta(), skip_until=RequiredUnitReady(UnitTypeId.SPIRE)),
                    Step(None, LingsBanes(), skip_until=RequiredUnitReady(UnitTypeId.BANELINGNEST)),
                    ZergUnit(UnitTypeId.ZERGLING),
                ),
                #skip=RequireCustom(lambda k: self.build_workers(k) and not self.max_workers_reached(k))
            ),
        )

    def build_workers(self, knowledge: Knowledge) -> bool:

        for zone in knowledge.zone_manager.expansion_zones:
            if zone.is_ours and zone.is_under_attack:
                return False

        return knowledge.game_analyzer.army_can_survive

    def max_workers_reached(self, knowledge: Knowledge) -> bool:
        count = 1
        for townhall in self.ai.townhalls:  # type: Unit
            count += townhall.ideal_harvesters
        for gas in self.ai.gas_buildings:  # type: Unit
            count += gas.ideal_harvesters

        if len(knowledge.unit_cache.own(UnitTypeId.DRONE)) >= count:
            return True

        return False