from typing import Union, Callable, List

from sc2 import UnitTypeId, Race
from sc2.ids.upgrade_id import UpgradeId
from sc2.unit import Unit
from sharpy.knowledges import KnowledgeBot, Knowledge
from sharpy.plans.zerg import *

class RoachAndHydras(BuildOrder):
    def __init__(self):

        self.hydras = ZergUnit(UnitTypeId.HYDRALISK, priority=True)
        self.roach = ZergUnit(UnitTypeId.ROACH)
        super().__init__(self.hydras, self.roach)

    async def execute(self) -> bool:
        self.hydras.to_count = self.get_count(UnitTypeId.ROACH) * 1.2
        return await super().execute()

class RoachHydraBuild(BuildOrder):
    def __init__(self):
        gas = SequentialList(
            [
                Step(
                    None,
                    StepBuildGas(2),
                    skip=RequiredGas(200),
                    skip_until=RequiredSupply(25, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(3),
                    skip=RequiredGas(200),
                    skip_until=RequiredSupply(40, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(4),
                    skip=RequiredGas(200),
                    skip_until=RequiredSupply(50, supply_type=SupplyType.Workers),
                ),
                Step(
                    RequiredMinerals(1000),
                    StepBuildGas(6),
                    skip=RequiredGas(200),
                    skip_until=RequiredSupply(50, supply_type=SupplyType.Workers),
                ),
                Step(
                    RequiredMinerals(2000),
                    StepBuildGas(8),
                    skip=RequiredGas(200),
                    skip_until=RequiredSupply(50, supply_type=SupplyType.Workers),
                ),
            ]
        )

        heavy_gas = SequentialList(
            [
                Step(
                    None,
                    StepBuildGas(2),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(20, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(3),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(30, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(4),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(40, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(5),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(50, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(6),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(60, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(7),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(65, supply_type=SupplyType.Workers),
                ),
                Step(
                    None,
                    StepBuildGas(8),
                    skip=RequiredGas(300),
                    skip_until=RequiredSupply(70, supply_type=SupplyType.Workers),
                ),
            ]
        )

        super().__init__(
            SequentialList(
                # Overlords
                Step(UnitExists(UnitTypeId.DRONE, 13, include_pending=True), ZergUnit(UnitTypeId.OVERLORD, 2, priority=True)),
                Step(UnitExists(UnitTypeId.DRONE, 20, include_pending=True), ZergUnit(UnitTypeId.OVERLORD, 3, priority=True)),
                AutoOverLord(),
            ),
            SequentialList(
                # Ling zealot defence
                Step(RequiredAny([EnemyUnitExists(UnitTypeId.ZERGLING, count=5), EnemyUnitExists(UnitTypeId.ZEALOT, count=5)]), ActBuilding(UnitTypeId.BANELINGNEST)),
                ZergUnit(UnitTypeId.BANELING, to_count=8),
                ActTech(UpgradeId.CENTRIFICALHOOKS),
            ),
            SequentialList(
                # Opener
                Step(RequiredSupply(17), ActExpand(2)),
                Step(
                    RequiredSupply(17),
                    PositionBuilding(UnitTypeId.SPAWNINGPOOL, DefensePosition.BehindMineralLineLeft, 0),
                ),
                StepBuildGas(1, RequiredSupply(16)),
                ActUnit(UnitTypeId.ZERGLING, UnitTypeId.LARVA, 4),
                ZergUnit(UnitTypeId.QUEEN, 2),
                Step(self.build_workers, None, skip=Time(8 * 60)),
                Step(Time(3 * 60), ActExpand(3)),
                ZergUnit(UnitTypeId.QUEEN, 4),
                Step(RequiredSupply(40, SupplyType.Workers), ActExpand(4)),
                ZergUnit(UnitTypeId.QUEEN, 6),
                Step(RequiredSupply(70, SupplyType.Workers), ActExpand(5)),
                ZergUnit(UnitTypeId.QUEEN, 8),
            ),
            SequentialList(
                # Workers
                ZergUnit(UnitTypeId.DRONE, 14),
                Step(self.build_workers, ZergUnit(UnitTypeId.DRONE, 80), skip=self.max_workers_reached),
            ),
            SequentialList(
                [
                    Step(RequiredAny([RequiredUnitReady(UnitTypeId.LAIR), RequiredUnitReady(UnitTypeId.HIVE)]), heavy_gas),
                    Step(RequiredAny([RequiredUnitReady(UnitTypeId.LAIR), RequiredUnitReady(UnitTypeId.HIVE)]), MorphOverseer(2)),
                ]
            ),
            SequentialList(
                Step(RequireCustom(lambda k: k.enemy_units_manager.enemy_cloak_trigger), MorphLair()),
                Step(RequiredUnitReady(UnitTypeId.LAIR), MorphOverseer(2)),
            ),
            SequentialList(
                # Tech
                Step(RequiredUnitReady(UnitTypeId.SPAWNINGPOOL), ActBuilding(UnitTypeId.ROACHWARREN)),
                Step(RequiredUnitReady(UnitTypeId.HATCHERY), MorphLair(), skip=RequiredUnitReady(UnitTypeId.HIVE)),
                Step(RequiredAll([RequiredUnitReady(UnitTypeId.ROACHWARREN), RequiredUnitReady(UnitTypeId.LAIR)]), ActTech(UpgradeId.GLIALRECONSTITUTION)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActBuilding(UnitTypeId.HYDRALISKDEN)),
                Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEMUSCULARAUGMENTS)),
                Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActTech(UpgradeId.EVOLVEGROOVEDSPINES)),
                ActBuilding(UnitTypeId.INFESTATIONPIT),
                MorphHive(),
            ),
            SequentialList(
                # Upgrades
                Step(RequiredUnitReady(UnitTypeId.HYDRALISKDEN), ActBuilding(UnitTypeId.EVOLUTIONCHAMBER, to_count=2)),
                Step(RequiredUnitReady(UnitTypeId.EVOLUTIONCHAMBER), ActTech(UpgradeId.ZERGMISSILEWEAPONSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.EVOLUTIONCHAMBER), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL1)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.ZERGMISSILEWEAPONSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.LAIR), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL2)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGMISSILEWEAPONSLEVEL3)),
                Step(RequiredUnitReady(UnitTypeId.HIVE), ActTech(UpgradeId.ZERGGROUNDARMORSLEVEL3)),
            ),
#            SequentialList(
                # Some standard air defence
#                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 40), PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=0)),
#                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 39), PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=1, to_count=2)),
#                Step(RequireCustom(lambda k: k.enemy_race != "zerg" and self.get_count(UnitTypeId.DRONE) > 38), PositionBuilding(UnitTypeId.SPORECRAWLER, DefensePosition.CenterMineralLine, to_base_index=2, to_count=3)),
#            ),
#            SequentialList(
                # Some vipers are good to have
#                Step(RequiredUnitReady(UnitTypeId.HIVE), ZergUnit(UnitTypeId.VIPER, to_count=3)),
#            ),
            Step(
                None,
                SequentialList(
                    # Units
                    Step(None, RoachAndHydras(), skip_until=RequiredUnitReady(UnitTypeId.HYDRALISKDEN)),
                    Step(None, ZergUnit(UnitTypeId.ROACH, priority=True), skip_until=RequiredUnitReady(UnitTypeId.ROACHWARREN)),
                    Step(None, PositionBuilding(UnitTypeId.SPINECRAWLER, DefensePosition.Entrance, to_count=3), skip_until=EnemyUnitExists(UnitTypeId.ROACH, 3)),
                    Step(None, PositionBuilding(UnitTypeId.SPINECRAWLER, DefensePosition.Entrance, to_count=2), skip_until=EnemyUnitExists(UnitTypeId.ZERGLING, 8)),
                    ZergUnit(UnitTypeId.ZERGLING),
                ),
            ),
        )

    def build_workers(self, knowledge: Knowledge) -> bool:
        
        for zone in knowledge.zone_manager.expansion_zones:
            if zone.is_ours and zone.is_under_attack:
                return False

        return knowledge.game_analyzer.army_can_survive

    def max_workers_reached(self, knowledge: Knowledge) -> bool:
        count = 1
        for townhall in self.ai.townhalls:  # type: Unit
            count += townhall.ideal_harvesters
        for gas in self.ai.gas_buildings:  # type: Unit
            count += gas.ideal_harvesters

        if len(knowledge.unit_cache.own(UnitTypeId.DRONE)) >= count:
            return True

        return False