from sc2 import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans.zerg import *


def build_workers(knowledge: "Knowledge") -> bool:
    for zone in knowledge.zone_manager.expansion_zones:
        if zone.is_ours and zone.is_under_attack:
            return knowledge.game_analyzer.army_can_survive

    if len(knowledge.unit_cache.own(UnitTypeId.DRONE)) >= 30:
        return knowledge.game_analyzer.army_can_survive

    return True

def max_workers_reached(knowledge: "Knowledge") -> bool:
    count = 1
    for townhall in knowledge.ai.townhalls:  # type: Unit
        if townhall.is_ready:
            count += townhall.ideal_harvesters
        elif townhall.is_ready > 0.6:
            count += 16
    for gas in knowledge.ai.gas_buildings:  # type: Unit
        count += gas.ideal_harvesters

    # Don't overbuild drones if enemy has less bases
    # To prepare defence for one base attacks and all ins
    if count > 25 * len(knowledge.enemy_townhalls):
        count = 25 * len(knowledge.enemy_townhalls)

    if len(knowledge.unit_cache.own(UnitTypeId.DRONE)) >= count:
        return True

    return False


def max_workers_almost_reached(knowledge: "Knowledge") -> bool:
    count = 0
    for townhall in knowledge.ai.townhalls:  # type: Unit
        if townhall.is_ready:
            count += townhall.ideal_harvesters
        elif townhall.is_ready > 0.6:
            count += 16
    for gas in knowledge.ai.gas_buildings:  # type: Unit
        count += gas.ideal_harvesters

    if len(knowledge.unit_cache.own(UnitTypeId.DRONE)) >= count - 10:
        return True

    return False


def skip_new_base(knowledge: "Knowledge") -> bool:
    if 2 < knowledge.ai.already_pending(UnitTypeId.HATCHERY):
        return True

    if (knowledge.available_mineral > 800 \
        or max_workers_almost_reached(knowledge)) \
            and len(knowledge.unit_cache.own(UnitTypeId.DRONE)) > 28:
        return not knowledge.game_analyzer.army_can_survive

    return True


def is_not_lategame(knowledge: "Knowledge"):
    return len(knowledge.unit_cache.own({UnitTypeId.HIVE, UnitTypeId.INFESTATIONPIT})) == 0

def is_not_midgame(knowledge: "Knowledge"):
    return not (
            is_not_lategame(knowledge) and
            len(knowledge.unit_cache.own(
                {UnitTypeId.LAIR, UnitTypeId.ROACHWARREN, UnitTypeId.HYDRALISKDEN, UnitTypeId.BANELINGNEST}
            )) > 0)

def is_not_earlygame(knowledge: "Knowledge"):
    return not (is_not_midgame(knowledge) and is_not_lategame(knowledge))



ranged_upgrades = SequentialList(
    Step(UnitReady(UnitTypeId.EVOLUTIONCHAMBER), Tech(UpgradeId.ZERGMISSILEWEAPONSLEVEL1)),
    Step(UnitReady(UnitTypeId.EVOLUTIONCHAMBER), Tech(UpgradeId.ZERGGROUNDARMORSLEVEL1)),
    Step(UnitReady(UnitTypeId.LAIR), Tech(UpgradeId.ZERGMISSILEWEAPONSLEVEL2)),
    Step(UnitReady(UnitTypeId.LAIR), Tech(UpgradeId.ZERGGROUNDARMORSLEVEL2)),
    Step(UnitReady(UnitTypeId.HIVE), Tech(UpgradeId.ZERGMISSILEWEAPONSLEVEL3)),
    Step(UnitReady(UnitTypeId.HIVE), Tech(UpgradeId.ZERGGROUNDARMORSLEVEL3)),
)

melee_upgrades = SequentialList(
    Step(UnitReady(UnitTypeId.EVOLUTIONCHAMBER), Tech(UpgradeId.ZERGMELEEWEAPONSLEVEL1)),
    Step(UnitReady(UnitTypeId.EVOLUTIONCHAMBER), Tech(UpgradeId.ZERGGROUNDARMORSLEVEL1)),
    Step(UnitReady(UnitTypeId.LAIR), Tech(UpgradeId.ZERGMELEEWEAPONSLEVEL2)),
    Step(UnitReady(UnitTypeId.LAIR), Tech(UpgradeId.ZERGGROUNDARMORSLEVEL2)),
    Step(UnitReady(UnitTypeId.HIVE), Tech(UpgradeId.ZERGMELEEWEAPONSLEVEL3)),
    Step(UnitReady(UnitTypeId.HIVE), Tech(UpgradeId.ZERGGROUNDARMORSLEVEL3)),
)

