from micro.generic_micro import GenericMicro
from sc2.ids.effect_id import EffectId
from sc2.position import Point2
from sharpy.managers.combat2 import Action, MoveType
from sc2 import AbilityId, Optional, Set, UnitTypeId
from sc2.unit import Unit


class MicroRavagers(GenericMicro):
    def unit_solve_combat(self, unit: Unit, current_command: Action) -> Action:
        shuffler = unit.tag % 10

        if not self.cd_manager.is_ready(unit.tag, AbilityId.EFFECT_CORROSIVEBILE):
            return super().unit_solve_combat(unit, current_command)

        if self.engaged_power.power > 10:
            best_score = 1
            target: Optional[Unit] = None
            enemy: Unit

            for enemy in self.enemies_near_by:
                d = enemy.distance_to(unit)
                if 2 < d < 9 + 3 and unit.type_id not in [UnitTypeId.DRONE, UnitTypeId.ZERGLING, UnitTypeId.BANELING, UnitTypeId.ZEALOT, UnitTypeId.EXTRACTOR, UnitTypeId.PROBE, UnitTypeId.SCV, UnitTypeId.MEDIVAC, UnitTypeId.BROODLING]:
                    score = d * 0.4 + enemy.radius - enemy.movement_speed * 2 + self.unit_values.power(enemy) * 0.2 + self.unit_values.ground_range(enemy)
                    score += 0.25 * (enemy.tag % (shuffler + 2))

                    if score > best_score:
                        target = enemy
                        best_score = score

            if target is not None:
                # Find where on the target to hit
                position = target.position

                if target.movement_speed < 0.1:
                    best_score2 = 1
                    target2: Optional[Unit] = None
                    enemy2: Unit

                    for enemy2 in self.cache.enemy_in_range(target.position, target.radius + 4):
                        d = enemy2.distance_to(target)
                        if d < 0.5:
                            score = d * 0.2 - enemy.movement_speed + enemy.radius + self.unit_values.power(enemy)

                            if score > best_score2:
                                target2 = enemy2
                                best_score2 = score

                    if target2 is not None:
                        position = position.towards(target2.position, target.radius)
                    else:
                        position = position.towards(self.enemies_near_by.center, target.radius)


                # Bile time to land is about 2.5s
                elif self.move_type == MoveType.DefensiveRetreat:
                    position = position.towards(unit.position, -target.radius)
                elif self.move_type == MoveType.PanicRetreat:
                    position = position.towards(unit.position, -target.radius - target.movement_speed * 1.5)

                elif self.move_type == MoveType.Push:
                    position = position.towards(unit.position, +target.radius)

                elif self.move_type == MoveType.Assault:
                    position = position.towards(unit.position, +target.radius)

                return Action(position, False, AbilityId.EFFECT_CORROSIVEBILE)

        return super().unit_solve_combat(unit, current_command)
