from sc2 import UnitTypeId, List
from sc2.units import Units
from sharpy.managers import UnitCacheManager
from sharpy.plans.acts import ActBase


class ActCrawlerAttack(ActBase):
    units: Units

    def __init__(self, unit_count: int):
        self.unit_count = unit_count
        self.building_count = 0
        self.units: List[Units] = []
        self.tags: List[int] = []
        self.buildings: List[int] = []
        super().__init__()

    async def start(self, knowledge: "Knowledge"):
        await super().start(knowledge)
        self.units = Units([], self.ai)

    async def execute(self) -> bool:
        unit_cache: UnitCacheManager = self.knowledge.unit_cache
        attack_pos = self.knowledge.expansion_zones[-1].center_location
        if len(self.units) == 0:
            self.units = unit_cache.own(UnitTypeId.DRONE).closest_n_units(attack_pos, 9)
            for unit in self.units:
                self.do(unit.move(attack_pos))
                self.tags.append(unit.tag)
            return True

        self.units = unit_cache.all_own.tags_in(self.tags).closer_than(3, attack_pos)
        for unit in self.units:
            if self.building_count < self.unit_count:
                self.do(unit.build(UnitTypeId.SPINECRAWLER, await self.knowledge.ai.find_placement(UnitTypeId.SPINECRAWLER, unit.position)))
                self.building_count += 1
                continue

            self.do(unit.attack(attack_pos))
        return True
