from random import shuffle
from typing import List

from sc2.position import Point2
from sharpy.managers.roles import UnitTask
from sharpy.knowledges import Knowledge
from sharpy.plans.acts import ActBase
from sc2 import UnitTypeId


class LingScoutAllExpos(ActBase):
    def __init__(self):
        self.scout: int = None
        self.route: List[Point2] = []
        super().__init__()

    async def start(self, knowledge: Knowledge):
        return await super().start(knowledge)

    def get_scout(self, start):
        lings = self.roles.free_units({UnitTypeId.ZERGLING})
        if len(lings) > 0:
            scout = lings.closest_to(start)
            return scout
        return None

    async def execute(self) -> bool:
        knowledge: Knowledge = self.knowledge

        if self.scout and knowledge.unit_cache.by_tag(self.scout):

            # If it is idle scout again
            scout_unit = knowledge.unit_cache.by_tag(self.scout)
            if scout_unit.is_idle:
                self.route = [expo.center_location for expo in knowledge.expansion_zones if not expo.enemy_townhall and not expo.our_townhall]
                shuffle(self.route)
                self.knowledge.print("Idle scout, adding more work")
                self.knowledge.print("Scouting " + self.route.__str__())
                for pos in self.route:
                    self.do(scout_unit.move(position=pos, queue=True))

            return True

        self.route = [expo.center_location for expo in knowledge.expansion_zones if not expo.enemy_townhall and not expo.our_townhall]
        shuffle(self.route)
        if len(self.route) == 0:
            self.knowledge.print("All expos accounted for")
            return True

        scout_unit = self.get_scout(self.route[0])
        if not scout_unit:
            self.knowledge.print("No available lings")
            return True

        self.knowledge.print("New scout aquired tag: " + str(scout_unit.tag))
        self.knowledge.print("Scouting " + self.route.__str__())
        self.roles.set_task(UnitTask.Scouting, scout_unit)
        for pos in self.route:
            self.do(scout_unit.move(pos, True))
        self.scout = scout_unit.tag

        return True
