from typing import List, Optional, Dict

from sc2.position import Point2
from sharpy.managers.roles import UnitTask
from sharpy.knowledges import Knowledge
from sharpy.managers.game_analyzer import GameAnalyzer
from sharpy.plans.acts import ActBase
from sc2 import UnitTypeId, AbilityId
from sc2.ids.buff_id import BuffId
from sc2.unit import Unit
from sc2.units import Units


class OvieScout(ActBase):
    def __init__(self, interval = 3 * 60):
        self.scouts: List[int] = []
        self.routes: List[List[Point2]] = []
        self.phase = 0
        self.interval = interval
        super().__init__()

    async def start(self, knowledge: Knowledge):
        await super().start(knowledge)

        target1 = self.knowledge.expansion_zones[-1].center_location.towards_with_random_angle(
            self.knowledge.expansion_zones[-2].center_location, -30)
        target2 = self.knowledge.expansion_zones[-1].center_location
        target3 = self.knowledge.expansion_zones[-2].center_location.towards_with_random_angle(
            self.knowledge.expansion_zones[-1].center_location, -35)

        # self.routes.append([target1, target2, target3])
        self.routes.append([target3, target2, target1])

    def get_scout(self, start):
        overlords = self.cache.own({ UnitTypeId.OVERLORD }).tags_not_in(self.scouts)
        if len(overlords) > 0:
            scout = overlords.closest_to(start)
            return scout
        return None

    async def execute(self) -> bool:
        if self.phase == 3:
            if self.ai.time % self.interval < 1 and not self.ai.time == 0:
                self.phase = 0
                self.scouts = []
            else:
                # Micro ovie
                for n in range(len(self.scouts)):
                    scout = self.cache.by_tag(self.scouts[n])
                    if scout:
                        enemies = self.cache.enemy_in_range(scout.position, 12).filter(lambda u: u.can_attack_air)
                        if len(enemies) > 4:
                            enemy = enemies.closest_to(scout)
                            if enemy.air_range + 2 > scout.distance_to(enemy):
                                self.do(scout.move(scout.position.towards(enemy.position, -3)))
                            else:
                                self.do(scout.move(self.routes[n][-1]))

            return True

        if self.phase == 0 and len(self.scouts) < len(self.routes):
            overlords = self.cache.own({UnitTypeId.OVERLORD})

            if len(overlords) >= len(self.routes):
                for n in range(len(self.routes)):
                    scout = overlords.closest_to(self.routes[0][n])
                    self.scouts.append(scout.tag)
                    overlords = overlords.tags_not_in(self.scouts)

                    #if len(self.cache.own({UnitTypeId.LAIR}).ready) > 0:
                    #    self.do(scout(AbilityId.MORPH_OVERSEER))
                self.phase = 1

        if self.phase == 1 and len(self.scouts) == len(self.routes):
            for n in range(len(self.routes)):
                scout = self.cache.by_tag(self.scouts[n])
                if scout and scout.is_idle:
                    self.do(scout.move(self.routes[n][0], queue=True))

                if not scout:
                    self.scouts[n] = self.get_scout(self.routes[n][0]).tag

            self.phase = 2

        if self.phase == 2:
            all_in_place = True
            for n in range(len(self.routes)):
                scout = self.cache.by_tag(self.scouts[n])
                if scout and (not scout.is_idle or scout.is_moving):
                    all_in_place = False

                if not scout:
                    self.scouts[n] = self.get_scout(self.routes[n][0])

            if all_in_place:
                for n in range(len(self.routes)):
                    scout = self.cache.by_tag(self.scouts[n])
                    for target in self.routes[n]:
                        self.do(scout.move(target, queue=True))
            self.phase = 3

        return True
