from sc2 import UnitTypeId
from sc2.units import Units
from sharpy.knowledges import Knowledge
from sharpy.managers.roles import UnitTask
from sharpy.plans.acts import ActBase
from sharpy.plans.tactics import PlanZoneDefense, PlanZoneAttack
from sharpy.plans.tactics.zone_attack import AttackStatus


class PlanZoneCounterAttack(ActBase):
    def __init__(self, zone_defence: PlanZoneDefense, zone_attack: PlanZoneAttack):
        super().__init__()
        self.zone_defence = zone_defence
        self.zone_attack = zone_attack

        self.attackStatus = 0

    async def start(self, knowledge: Knowledge):
        await super().start(knowledge)

    async def execute(self) -> bool:
        if self.attackStatus == 0:
            for i in range(0, len(self.knowledge.expansion_zones)):
                zone: "Zone" = self.knowledge.expansion_zones[i]

                if len(zone.known_enemy_units) > 5 and zone.is_ours:
                    self._start_attack(zone)
                    break

        elif self.zone_attack.status != AttackStatus.Attacking:
                self.attackStatus = 0
                self.print(f"Counterattack stopped.")

        return True

    def _start_attack(self, zone: "Zone"):
        self.attackStatus = 1

        units: Units = self.knowledge.roles.free_units.exclude_type(UnitTypeId.QUEEN)
        if len(units) < 10:
            if self.knowledge.ai.supply_used > 100:
                for type_id in [UnitTypeId.MUTALISK, UnitTypeId.ZERGLING, UnitTypeId.ROACH, UnitTypeId.HYDRALISK]:
                    own_units = self.cache.own(type_id).furthest_n_units(zone.assaulting_enemies.center, 20)
                    units.extend(own_units)

                    if len(units) > 20:
                        break

        if len(units) < 10:
            self.print(f"Too few units to counter")
            return True

        self.knowledge.roles.set_tasks(UnitTask.Attacking, units)
        self.zone_attack.status = AttackStatus.Attacking
        self.print(f"Counterattack started with " + str(len(units)) + " units")
