from typing import List, Optional, Dict

from sc2.position import Point2
from sharpy.managers.roles import UnitTask
from sharpy.knowledges import Knowledge
from sharpy.managers.game_analyzer import GameAnalyzer
from sharpy.plans.acts import ActBase
from sc2 import UnitTypeId, AbilityId
from sc2.ids.buff_id import BuffId
from sc2.unit import Unit
from sc2.units import Units


class ScoutTarget():
    def __init__(self, pos: Point2):
        self.pos = pos
        self.last_scouted = 1
        self.scout_interval = 100
        self.unit_scouting = None
        self.time = 1
        return

    @property
    def priority(self):
        # Prioritize when not scouted in a long time
        return self.should_scout

    @property
    def should_scout(self):
        return self.last_scouted + self.scout_interval - self.time

    def update(self, time):
        self.time = time


class Scout(ActBase):
    def __init__(self):
        self.targets: List[ScoutTarget] = []
        self.scouts: Dict[int, ScoutTarget] = {}
        super().__init__()

    async def start(self, knowledge: Knowledge):
        return await super().start(knowledge)

    async def execute(self) -> bool:
        overlords = self.cache.own(UnitTypeId.OVERLORD)
        scouts = self.roles.all_from_task(UnitTask.Scouting)
        overlords = overlords.tags_not_in(scouts.tags)

        knowledge: Knowledge = self.knowledge

        analyzer: GameAnalyzer = knowledge.game_analyzer

        if analyzer.army_at_least_small_disadvantage:
            return True

        if len(self.targets) == 0:
            expansions = self.knowledge.expansion_zones

            self.targets.append(ScoutTarget(expansions[-2].center_location))
            self.targets.append(ScoutTarget(expansions[-1].center_location))
            self.targets.append(ScoutTarget(knowledge.ai.game_info.map_center))
            self.targets.append(ScoutTarget(expansions[-3].center_location))
            self.targets.append(ScoutTarget(expansions[-4].center_location))
            self.targets.append(ScoutTarget(expansions[-5].center_location))
            self.targets.append(ScoutTarget(expansions[-6].center_location))

        targets = sorted(self.targets, key=lambda t: t.priority)

        for target in targets:

            target.update(self.ai.time)

            if target.unit_scouting is None or not knowledge.lost_units_manager..cache.by_tag(target.unit_scouting.tag).health == 0:
                if len(overlords):
                    unit = overlords.first
                else:
                    units: Units = self.roles.all_from_task(UnitTask.Scouting).filter(lambda u: u.type_id == UnitTypeId.ZERGLING and u.is_idle)
                    if units.empty:
                        continue
                    unit = units.first

                knowledge.print(f"Add scouting unit: {unit.name}", stats=False)
                self.scouts[unit.tag] = target
                target.unit_scouting = unit
                self.roles.set_task(UnitTask.Scouting, unit)
                scouts = self.roles.all_from_task(UnitTask.Scouting)
                overlords = overlords.tags_not_in(scouts.tags)

            if target.priority > 0 and target.unit_scouting.is_idle:
                knowledge.print(f"Scouting: {target.pos}", stats=False)
                target.last_scouted = self.ai.time
                self.do(target.unit_scouting.move(target.pos))
                self.do(target.unit_scouting.move(target.pos.towards(target.unit_scouting.position, 20), queue=True))

        return True
