from sc2 import UnitTypeId
from sharpy.managers.roles import UnitTask
from sharpy.plans.acts import ActBase


class MineMineralWall(ActBase):
    def __init__(self):
        super().__init__()

        self.worker = None

    async def start(self, knowledge: "Knowledge"):
        await super().start(knowledge)

    def get_worker(self, target):
        workers = self.cache.own({UnitTypeId.DRONE})
        if len(workers) > 0:
            return workers.closest_to(target)
        return None

    def get_mineral_wall(self, target):
        walls = self.ai.mineral_field({UnitTypeId.MINERALFIELD450, UnitTypeId.MINERALFIELD750}) \
            .further_than(10, target) \
            .closer_than(40, target)
        if len(walls) > 0:
            return walls.closest_to(target)
        return None

    async def execute(self) -> bool:
        wall = self.get_mineral_wall(self.knowledge.ai.start_location)

        if wall and self.knowledge.iteration % 300 == 0:
            self.worker = self.get_worker(wall)

        if wall and self.knowledge.iteration % 20 == 0:
            if not self.worker or not self.cache.by_tag(self.worker.tag):
                self.worker = self.get_worker(wall)

            if not self.worker:
                return True

            if self.cache.by_tag(self.worker.tag).is_carrying_resource:
                self.do(self.worker.smart(self.cache.own({ UnitTypeId.HATCHERY, UnitTypeId.LAIR, UnitTypeId.HIVE }).closest_to(self.worker)))

            else:
                self.do(self.worker.gather(wall))

        else:
            if not self.worker:
                return True
            self.roles.set_task(UnitTask.Idle, self.worker)

        return True
